==> kom%2B%2B-0.3.tar.gz: kom++-0.3/libkom++/text.cc <==
// Fetch-on-demand cache for texts.
//
// $Id: text.cc,v 1.27 1994/08/11 23:23:08 ceder Exp $
// Copyright (C) 1994  Per Cederqvist and Inge Wallin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include <iostream.h>

#include "text.h"
#include "LyStr.h"
#include "array.h"
#include "person.h"
#include "conference.h"
#include "text-mass-cache.h"

#include "get_text_q.h"
#include "mark_as_read_q.h"
#include "mark_text_q.h"
#include "unmark_text_q.h"


// ================================================================
// ================================================================
//                            Class Text


// ================================================================ 
//                    Constructors and Destructors                  


Text::Text()
{
}


Text::Text(Text_no t, 
	   Text_stat_cache *tstat_cache,
	   Text_mass_cache *tmass_cache)
    : tno(t), ts_cache(tstat_cache), tm_cache(tmass_cache)
{
}
 
Text::Text(const Text &t)
    : tno(t.tno), ts_cache(t.ts_cache), tm_cache(t.tm_cache)
{
}


Text::~Text()
{
}


// ================================================================ 
//

const Text & 
Text::operator = (const Text &t)
{
    tno = t.tno;
    ts_cache = t.ts_cache;
    tm_cache = t.tm_cache;
    return *this;
}

Status
Text::prefetch() const
{
    return ts_cache->prefetch(tno);
}

Bool
Text::valid() const
{
    // return ts_cache->get(tno).valid();
    Bool GCCBUG = ts_cache->get(tno).valid();
    return GCCBUG;
}

int
Text::errno() const
{
    // return ts_cache->get(tno).errno();
    int GCCBUG = ts_cache->get(tno).errno();
    return GCCBUG;
}

Text_no
Text::text_no() const
{
    return tno;
}

int
Text::prefetch_for_display(int max_questions, Session *sess) const
{
    // +++ Not yet implemented.
    return 0;
}

struct tm
Text::creation_time() const
{
    assert(valid());
    //return ts_cache->get(tno).creation_time();
    struct tm GCCBUG = ts_cache->get(tno).creation_time();
    return GCCBUG;
}

Pers_no
Text::author() const
{
    assert(valid());
    // return ts_cache->get(tno).author();
    Pers_no GCCBUG = ts_cache->get(tno).author();
    return GCCBUG;
}

long       
Text::num_lines() const
{
    assert(valid());
    // return ts_cache->get(tno).num_lines();
    long GCCBUG = ts_cache->get(tno).num_lines();
    return GCCBUG;
}

long       
Text::num_chars() const
{
    assert(valid());
    return ts_cache->get(tno).num_chars();
}

int         
Text::num_marks() const
{
    assert(valid());
    // return ts_cache->get(tno).num_marks();
    int GCCBUG = ts_cache->get(tno).num_marks();
    return GCCBUG;
}

int
Text::num_recipients() const
{
    assert(valid());
    int GCCBUG = ts_cache->get(tno).num_recipients();
    return GCCBUG;
    // return ts_cache->get(tno).num_recipients();
}

Recipient* 
Text::recipient(int i) const
{
    assert(valid());
    // return ts_cache->get(tno).recipient(i);
    Recipient *GCCBUG = ts_cache->get(tno).recipient(i);
    return GCCBUG;
}

int
Text::num_comment_in() const
{
    assert(valid());
    //return ts_cache->get(tno).num_comment_in();
    int GCCBUG = ts_cache->get(tno).num_comment_in();
    return GCCBUG;
}

Text_no	
Text::comment_in(int i) const
{
    assert(valid());
    // return ts_cache->get(tno).comment_in(i);
    Text_no GCCBUG = ts_cache->get(tno).comment_in(i);
    return GCCBUG;
}

int
Text::num_footnote_in() const
{
    assert(valid());
    // return ts_cache->get(tno).num_footnote_in();
    int GCCBUG = ts_cache->get(tno).num_footnote_in();
    return GCCBUG;
}

Text_no
Text::footnote_in(int i) const
{
    assert(valid());
    // return ts_cache->get(tno).footnote_in(i);
    Text_no GCCBUG = ts_cache->get(tno).footnote_in(i);
    return GCCBUG;
}

int		
Text::num_uplinks() const
{
    assert(valid());
    // return ts_cache->get(tno).num_uplinks();
    int GCCBUG = ts_cache->get(tno).num_uplinks();
    return GCCBUG;
}

Uplink*
Text::uplink(int i)	 const
{
    assert(valid());
    // return ts_cache->get(tno).uplink(i);
    Uplink *GCCBUG = ts_cache->get(tno).uplink(i);
    return GCCBUG;
}

//+++ Return a LyStr * instead??
LyStr
Text::text_mass(long start, long end) const
{
    assert(valid());
    LyStr result = *(tm_cache->get(tno));
    result.remove_first(start);
    return result.substr(end);
}

LyStr
Text::subject() const
{
    LyStr str = text_mass(0, 256);

    for (int i=0; i < str.strlen() && str[i] != '\n'; i++)
	continue;

    return str.substr(i);
}

LyStr
Text::body(long start, long end) const
{
    LyStr str = text_mass();

    for (int i=0; i < str.strlen() && str[i] != '\n'; i++)
	continue;
    str.remove_first(i);
    if (str.strlen() > 0 && str[0] == '\n')
	str.remove_first(1);
    str.remove_first(start);
    return str.substr(end);
}


Status
Text::mark_as_read(Conf_no cno)
{
    Text_stat   textstat;
    int         i;

    assert(valid());
    textstat = ts_cache->get(tno);
    
    // If cno=0, then mark the text as read in all conferences.
    if (cno == 0) {
	Array<mark_as_read_question *>   mark_q_arr(textstat.num_recipients());
	Status                           return_status;

	for (i = 0; i < textstat.num_recipients(); ++i) {
	    mark_q_arr[i] 
		= new mark_as_read_question(ts_cache->connection(),
					    textstat.recipient(i)->recipient(),
					    textstat.recipient(i)->local_no());
	}

	return_status = st_ok;
	for (i = 0; i < textstat.num_recipients(); ++i) {
	    if (return_status == st_ok)
		return_status = mark_q_arr[i]->receive();
	    else
		(void) mark_q_arr[i]->receive();

	    delete mark_q_arr[i];
	}
	return return_status;

    } else {
	// Search the recipients for the one to mark as read.
	for (i = 0; i < textstat.num_recipients(); ++i) {
	    mark_as_read_question  * mark_q;
	    Recipient              * recip;

	    recip = textstat.recipient(i);
	    if (recip->recipient() == cno) {
		mark_q = new mark_as_read_question(ts_cache->connection(),
						   recip->recipient(),
						   recip->local_no());
		return mark_q->receive();
	    }
	}
    }

    return st_ok;
}



Status
Text::mark(unsigned char mark_type)
{
    // +++FIXME: This loses the error number if it fails. Bad, bad.
    // This comment also applies to Text::unmark, and probably other
    // methods as well.
    mark_text_question   mark_q(ts_cache->connection(), tno, mark_type);
    return mark_q.receive();
}

Status
Text::unmark()
{
    unmark_text_question   unmark_q(ts_cache->connection(), tno);
    return unmark_q.receive();
}

Local_text_no
Text::local_no_recipient(Conf_no conf_no)
{
    for (int i=0; i < num_uplinks(); i++)
	if (recipient(i)->recipient() == conf_no)
	    return recipient(i)->local_no();
	
    return 0;
}

// ================================================================
// ================================================================
//                            Class Texts


// ================================================================ 
//                    Constructors and Destructors                  


Texts::Texts()
    : connection(NULL), ts_cache(NULL), tm_cache(NULL)
{
}
Texts::Texts(Connection *conn)
    : connection(conn), ts_cache(new Text_stat_cache(conn)),
      tm_cache(new Text_mass_cache(conn))
{
}
 

Texts::Texts(const Texts &text)
{
    abort();
}


Texts::~Texts()
{
    if (ts_cache)
	delete ts_cache;
    if (tm_cache)
	delete tm_cache;
}


// ================================================================ 
//

//const Texts & 
//Texts::operator = (const Texts &)
//{
//    cerr << "Operator= for class Texts is not yet implemented\n";
//    abort();
//}

void
Texts::set_connection(Connection *conn)
{
    assert(connection == NULL);
    connection = conn;
    ts_cache = new Text_stat_cache(conn);
    tm_cache = new Text_mass_cache(conn);
}

Text
Texts::operator [](Text_no textno)
{
    assert(ts_cache != NULL);
    return Text(textno, ts_cache, tm_cache);
}

Text_stat_cache *
Texts::read_text_stat_cache() const
{
    return ts_cache;
}

Text_mass_cache *
Texts::read_text_mass_cache() const
{
    return tm_cache;
}

==> kom%2B%2B-0.5.tar.gz: kom++-0.5/libkom++/text.cc <==
// Fetch-on-demand cache for texts.
//
// $Id: text.cc,v 1.35 1996/01/07 17:55:48 inge Exp $
// Copyright (C) 1994  Per Cederqvist and Inge Wallin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <iostream.h>

#include "text.h"
#include "LyStr.h"
#include "array.h"
#include "person.h"
#include "conference.h"
#include "session.h"
#include "text-mass-cache.h"

#include "get_text_q.h"
#include "mark_as_read_q.h"
#include "mark_text_q.h"
#include "unmark_text_q.h"


// ================================================================
// ================================================================
//                            Class Text


// ================================================================ 
//                    Constructors and Destructors                  


Text::Text()
{
}


Text::Text(Text_no t, 
	   Text_stat_cache *tstat_cache,
	   Text_mass_cache *tmass_cache)
    : tno(t), ts_cache(tstat_cache), tm_cache(tmass_cache)
{
}
 
Text::Text(const Text &t)
    : tno(t.tno), ts_cache(t.ts_cache), tm_cache(t.tm_cache)
{
}


Text::~Text()
{
}


// ================================================================ 
//

const Text & 
Text::operator = (const Text &t)
{
    tno = t.tno;
    ts_cache = t.ts_cache;
    tm_cache = t.tm_cache;
    return *this;
}

Status
Text::prefetch() const
{
    return ts_cache->prefetch(tno);
}

int
Text::errno() const
{
    // return ts_cache->get(tno).errno();
    int GCCBUG = ts_cache->get(tno).errno();
    return GCCBUG;
}

bool
Text::present() const
{
    // return ts_cache->get(tno).present();
    bool GCCBUG = ts_cache->present(tno);
    return GCCBUG;
}

bool
Text::exists() const
{
    return (bool)(errno() == KOM_NO_ERROR);
}

Text_no
Text::text_no() const
{
    return tno;
}


//
// Prefetch everything that is needed by most clients to display this
// text, but issue at most MAX_QUESTIONS questions. Return the
// number of pending questions, or -1 if a hard error occurs.
//
int
Text::prefetch_for_display(int max_questions, Session *sess) const
{
    // Note: if this function becomes a "hot spot" it can be made more
    // efficient by introducing a "bool prefetch_for_display_done;"
    // flag which is set when this function return 0 (that is, when
    // everything is prefetched).  That bit could then be tested here:
    // if (prefetch_for_display_done) return 0;
    int q = 0;

    if (q >= max_questions)
	return q;
    
    // Step 1: Prefetch the text itself.
    switch (prefetch())
    {
    case st_ok:
	break;
    case st_error:
	return 0;
    case st_pending:
	return 1;
    }
    if (q >= max_questions)
	return q;
    if (sess->conferences()[author()].prefetch() == st_pending)
	q++;
    if (q >= max_questions)
	return q;
    for (int rcpt = 0; rcpt < num_recipients(); rcpt++)
    {
	Conference c = sess->conferences()[recipient(rcpt)->recipient()];
	if (c.prefetch() == st_pending)
	    q++;
	if (q >= max_questions)
	    return q;
    }

    for (int comm_in = 0; comm_in < num_comment_in(); comm_in++)
    {
	Text t = sess->texts()[comment_in(comm_in)];
	switch (t.prefetch())
	{
	case st_ok:
	    if (sess->conferences()[t.author()].prefetch() == st_pending)
		q++;
	    break;
	case st_error:
	    break;
	case st_pending:
	    q++;
	}
	if (q >= max_questions)
	    return q;
    }
	
    // FIXME: prefetch text_mass and footnotes and uplinks.
    return q;
}

struct tm
Text::creation_time() const
{
    assert(errno() == KOM_NO_ERROR);
    //return ts_cache->get(tno).creation_time();
    struct tm GCCBUG = ts_cache->get(tno).creation_time();
    return GCCBUG;
}

Pers_no
Text::author() const
{
    assert(errno() == KOM_NO_ERROR);
    // return ts_cache->get(tno).author();
    Pers_no GCCBUG = ts_cache->get(tno).author();
    return GCCBUG;
}

long       
Text::num_lines() const
{
    assert(errno() == KOM_NO_ERROR);
    // return ts_cache->get(tno).num_lines();
    long GCCBUG = ts_cache->get(tno).num_lines();
    return GCCBUG;
}

long       
Text::num_chars() const
{
    assert(errno() == KOM_NO_ERROR);
    return ts_cache->get(tno).num_chars();
}

int         
Text::num_marks() const
{
    assert(errno() == KOM_NO_ERROR);
    // return ts_cache->get(tno).num_marks();
    int GCCBUG = ts_cache->get(tno).num_marks();
    return GCCBUG;
}

int
Text::num_recipients() const
{
    assert(errno() == KOM_NO_ERROR);
    int GCCBUG = ts_cache->get(tno).num_recipients();
    return GCCBUG;
    // return ts_cache->get(tno).num_recipients();
}

Recipient* 
Text::recipient(int i) const
{
    assert(errno() == KOM_NO_ERROR);
    // return ts_cache->get(tno).recipient(i);
    Recipient *GCCBUG = ts_cache->get(tno).recipient(i);
    return GCCBUG;
}

int
Text::num_comment_in() const
{
    assert(errno() == KOM_NO_ERROR);
    //return ts_cache->get(tno).num_comment_in();
    int GCCBUG = ts_cache->get(tno).num_comment_in();
    return GCCBUG;
}

Text_no	
Text::comment_in(int i) const
{
    assert(errno() == KOM_NO_ERROR);
    // return ts_cache->get(tno).comment_in(i);
    Text_no GCCBUG = ts_cache->get(tno).comment_in(i);
    return GCCBUG;
}

int
Text::num_footnote_in() const
{
    assert(errno() == KOM_NO_ERROR);
    // return ts_cache->get(tno).num_footnote_in();
    int GCCBUG = ts_cache->get(tno).num_footnote_in();
    return GCCBUG;
}

Text_no
Text::footnote_in(int i) const
{
    assert(errno() == KOM_NO_ERROR);
    // return ts_cache->get(tno).footnote_in(i);
    Text_no GCCBUG = ts_cache->get(tno).footnote_in(i);
    return GCCBUG;
}

int		
Text::num_uplinks() const
{
    assert(errno() == KOM_NO_ERROR);
    // return ts_cache->get(tno).num_uplinks();
    int GCCBUG = ts_cache->get(tno).num_uplinks();
    return GCCBUG;
}

Uplink*
Text::uplink(int i)	 const
{
    assert(errno() == KOM_NO_ERROR);
    // return ts_cache->get(tno).uplink(i);
    Uplink *GCCBUG = ts_cache->get(tno).uplink(i);
    return GCCBUG;
}

//+++ Return a LyStr * instead??
LyStr
Text::text_mass(long start, long end) const
{
    assert(errno() == KOM_NO_ERROR);
    LyStr result = *(tm_cache->get(tno));
    result.remove_first(start);
    return result.substr(end);
}

LyStr
Text::subject() const
{
    LyStr str = text_mass(0, 256);

    int i;
    for (i=0; i < str.strlen() && str[i] != '\n'; i++)
	continue;

    return str.substr(i);
}

LyStr
Text::body(long start, long end) const
{
    LyStr str = text_mass();

    int i;
    for (i=0; i < str.strlen() && str[i] != '\n'; i++)
	continue;
    str.remove_first(i);
    if (str.strlen() > 0 && str[0] == '\n')
	str.remove_first(1);
    str.remove_first(start);
    return str.substr(end);
}


Status
Text::mark_as_read(Conf_no cno)
{
    Text_stat   textstat;
    int         i;

    assert(errno() == KOM_NO_ERROR);
    textstat = ts_cache->get(tno);
    
    // If cno=0, then mark the text as read in all conferences.
    if (cno == 0) {
	Array<mark_as_read_question *>   mark_q_arr(textstat.num_recipients());
	Status                           return_status;

	for (i = 0; i < textstat.num_recipients(); ++i) {
	    mark_q_arr[i] 
		= new mark_as_read_question(ts_cache->connection(),
					    textstat.recipient(i)->recipient(),
					    textstat.recipient(i)->local_no());
	}

	return_status = st_ok;
	for (i = 0; i < textstat.num_recipients(); ++i) {
	    if (return_status == st_ok)
		return_status = mark_q_arr[i]->receive();
	    else
		(void) mark_q_arr[i]->receive();

	    delete mark_q_arr[i];
	}
	return return_status;

    } else {
	// Search the recipients for the one to mark as read.
	for (i = 0; i < textstat.num_recipients(); ++i) {
	    mark_as_read_question  * mark_q;
	    Recipient              * recip;

	    recip = textstat.recipient(i);
	    if (recip->recipient() == cno) {
		mark_q = new mark_as_read_question(ts_cache->connection(),
						   recip->recipient(),
						   recip->local_no());
		return mark_q->receive();
	    }
	}
    }

    return st_ok;
}



Status
Text::mark(unsigned char mark_type)
{
    // +++FIXME: This loses the error number if it fails. Bad, bad.
    // This comment also applies to Text::unmark, and probably other
    // methods as well.
    mark_text_question   mark_q(ts_cache->connection(), tno, mark_type);
    return mark_q.receive();
}

Status
Text::unmark()
{
    unmark_text_question   unmark_q(ts_cache->connection(), tno);
    return unmark_q.receive();
}

Local_text_no
Text::local_no_recipient(Conf_no conf_no)
{
    for (int i=0; i < num_recipients(); i++)
	if (recipient(i)->recipient() == conf_no)
	    return recipient(i)->local_no();
	
    return 0;
}

// ================================================================
// ================================================================
//                            Class Texts


// ================================================================ 
//                    Constructors and Destructors                  


Texts::Texts(Connection *conn, Async_dispatcher *async_disp)
    : connection(conn), ts_cache(new Text_stat_cache(conn, async_disp)),
      tm_cache(new Text_mass_cache(conn))
{
}
 

Texts::~Texts()
{
    if (ts_cache)
	delete ts_cache;
    if (tm_cache)
	delete tm_cache;
}


// ================================================================ 
//

Text
Texts::operator [](Text_no textno)
{
    assert(ts_cache != NULL);
    return Text(textno, ts_cache, tm_cache);
}

Text_stat_cache *
Texts::read_text_stat_cache() const
{
    return ts_cache;
}

Text_mass_cache *
Texts::read_text_mass_cache() const
{
    return tm_cache;
}
